pub static MAIN_CPP_HBS: &str = include_str!("main.cpp.hbs");
pub static LIB_HPP_HBS: &str = include_str!("lib.hpp.hbs");
pub static LIB_CPP_HBS: &str = include_str!("lib.cpp.hbs");
pub static EXPORT_H_HBS: &str = include_str!("export.h.hbs");

pub static CONFIG_H_CM_HBS: &str = include_str!("config.h.cm.hbs");
pub static USER_CMAKE_HBS: &str = include_str!("user.cmake.hbs");
pub static PROJECT_CONFIG_CMAKE_IN_HBS: &str = include_str!("config.cmake.in.hbs");
pub static VERSION_H_IN_HBS: &str = include_str!("version.h.in.hbs");
pub static WORKSPACE_CMAKE_LISTS_TXT_HBS: &str = include_str!("workspace.cmake.hbs");
pub static PROJECT_CMAKE_LISTS_TXT_HBS: &str = include_str!("project.cmake.hbs");
pub static PORT_FILE_CMAKE_HBS: &str = include_str!("portfile.cmake.hbs");

pub static SETUP_ISS_HUBS: &str = include_str!("setup.iss.hbs");
