#pragma once

#ifndef BUILD_SHARED_LIBS_{{project_upper}}
    #define {{project_upper}}_API
#else
    #ifdef _WIN32
        #ifdef {{project_upper}}_EXPORTS
            #define {{project_upper}}_API __declspec(dllexport)
        #else
            #define {{project_upper}}_API __declspec(dllimport)
        #endif
    #elif defined(__GNUC__) || defined(__clang__)
        #define {{project_upper}}_API __attribute__((visibility("default")))
    #else
        #define {{project_upper}}_API
    #endif
#endif
