/* config compiler, os, header, library, symbol */

/* Define to 1 if you have the <stdlib.h> header file. */
#cmakedefine HAVE_STDLIB_H 1

/* Define to 1 if you have the `gettimeofday' function. */
#cmakedefine HAVE_GETTIMEOFDAY 1

/* Define to 1 if you have the `O_BINARY' symbol. */
#cmakedefine HAVE_O_BINARY 1
