; Script generated by the Inno Setup Script Wizard.
; SEE THE DOCUMENTATION FOR DETAILS ON CREATING INNO SETUP SCRIPT FILES!

[Setup]
; NOTE: The value of AppId uniquely identifies this application. Do not use the same AppId value in installers for other applications.
; (To generate a new GUID, click Tools | Generate GUID inside the IDE.)
AppId={{uuid}}
AppName={{name}}
AppVersion={{version}}
DefaultDirName={autopf64}\\{{name}}
UsePreviousAppDir=no
DisableDirPage=no
DefaultGroupName={{name}}
AllowNoIcons=yes
DisableFinishedPage=yes
;PrivilegesRequired=lowest
PrivilegesRequired=admin
OutputDir=.\\{{target}}
OutputBaseFilename={{name}}
;SetupIconFile=setup.ico
Uninstallable=yes
;UninstallDisplayIcon=setup.ico
UninstallDisplayName={{name}}
CreateUninstallRegKey=no
Compression=lzma/ultra64   
SolidCompression=yes
ArchitecturesAllowed=x64os arm64
ArchitecturesInstallIn64BitMode=x64os arm64
WizardStyle=modern
AlwaysRestart=no
RestartIfNeededByRun=no
VersionInfoProductTextVersion="v{{version}}"
VersionInfoProductVersion="{{version}}.0"
VersionInfoTextVersion="v{{version}}"
VersionInfoVersion="{{version}}.0"
VersionInfoProductName="{{name}} - Package {{version}}"


[Tasks]
Name: "desktopicon"; Description: "{cm:CreateDesktopIcon}"; GroupDescription: "{cm:AdditionalIcons}";


[Files]
Source: "{{target}}\\{{installed}}\\{{triplet}}\\{{bin}}\*"; DestDir: "{app}"; Flags: ignoreversion recursesubdirs createallsubdirs;
; NOTE: Don't use "Flags: ignoreversion" on any shared system files


[Icons]
Name: "{group}\\{{name}}"; Filename: "{app}\\{{name}}.exe"
Name: "{group}\{cm:UninstallProgram,{{name}}}"; Filename: "{uninstallexe}"
Name: "{autodesktop}\\{{name}}"; Filename: "{app}\\{{name}}.exe"; Tasks: desktopicon


[InstallDelete]
Type: filesandordirs; Name: "{app}"


[UninstallDelete]
Type: filesandordirs; Name: "{app}"
