# set cmake version required
cmake_minimum_required(VERSION {{cmake_version}})

# set project name
project ({{project}} C CXX)

# add workspace members
{{#each members as |member|}}
add_subdirectory({{member}})
{{/each}}
