#include "export.h"

#ifdef __cplusplus
extern "C" {
#endif

int {{project_upper}}_API test(int left, int right);

#ifdef __cplusplus
}
#endif

class {{project_upper}}_API Test
{
public:
    int add(int left, int right);
};
