#ifndef RS_CONTAINER_FFI_C_STR_API
#define RS_CONTAINER_FFI_C_STR_API

// bindings of rs_container_ffi/c_str.rs

// rust CString
extern void rust_c_str_drop(char *s);

#endif // RS_CONTAINER_FFI_C_STR_API
